"""Tests for our `configure` command."""

import unittest
from unittest.mock import patch, mock_open
from docopt import docopt
from io import StringIO
import paci.cli as paci
from paci.commands.configure import Configure

doc = paci.__doc__


class TestConfigure(unittest.TestCase):

    def test_prints_the_welcome_msg(self):
        configure = Configure({'configure': True, '--no-choice': False, '--silent': True, '--main-registry': False, '--fallback-registry': False})

        with patch('sys.stdout', new_callable=StringIO) as sysout:
            with patch("paci.helpers.display_helper.std_input", return_value='') as m:
                configure.run()

        self.assertIn('Lets configure a new settings.yml for paci!', sysout.getvalue())

if __name__ == '__main__':
    unittest.main()



"""The list command."""

import os
from paci.helpers import display_helper, cache_helper
from termcolor import colored
from .base import Base


class List(Base):
    """Lists packages!"""

    def run(self):

        # List all installed packages
        if self.options["--installed"]:
            if not os.path.exists(self.index_file):
                print("No packages installed yet!")
                exit(1)

            print(colored("Displaying all installed packages...\n", 'yellow', attrs=['bold']))
            display_helper.print_list(["Name", "Version", "Description"], self.index.get_installed())

        # List all available packages
        if not self.options["--installed"]:
            print(colored("Displaying all available packages...\n", 'yellow', attrs=['bold']))

            result = cache_helper.get_pkgs(self.settings["paci"]["registry"], self.repo_cache)
            if result:
                display_helper.print_list(["Name", "Version", "Description", "Registry"], result)



"""Helper to output stuff"""

from tabulate import tabulate
import os


def print_list(header, entries):
    """Prints out a list"""
    print(tabulate(fix_descriptions(entries), header, tablefmt="presto"))


def print_table(entries):
    """Prints out a table"""
    print(tabulate(fix_descriptions(entries), tablefmt="plain"))


def std_input(text, default):
    """Get input or return default if none is given."""
    return input(text.format(default)) or default


def fix_descriptions(entries):
    """Fixes the description to fit into the terminal"""

    clean_entries = []
    ml = get_max_desc_width(get_longest_list(entries))

    for entry in entries:
        clean_entry = entry
        max_value = max(entry, key=len)
        for idx, val in enumerate(entry):
            if val is max_value:
                clean_entry[idx] = entry[idx][:ml] + (entry[idx][ml:] and ' [..]')
        clean_entries.append(clean_entry)

    return clean_entries


def get_longest_list(entries):
    max_list = ['']*len(entries[0])
    for entry in entries:
        for idx, val in enumerate(entry):
            if len(val) > len(max_list[idx]):
                max_list[idx] = val
    return max_list


def get_max_desc_width(lst):
    _, columns = os.popen('stty size', 'r').read().split()
    length = int(columns)
    max_value = max(lst, key=len)
    for val in lst:
        if val is not max_value:
            length -= len(val)

    return length - 20
